# Copyright (c) 2015-2016 Hewlett-Packard Enterprise Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

GLANCE_IMAGE = "OS::Glance::Image"
GLANCE_METADEF = "OS::Glance::Metadef"

CINDER_SNAPSHOT = "OS::Cinder::Snapshot"
CINDER_VOLUME = "OS::Cinder::Volume"

DESIGNATE_ZONE = "OS::Designate::Zone"
DESIGNATE_RECORDSET = "OS::Designate::RecordSet"

NEUTRON_NETWORK = "OS::Neutron::Net"
NEUTRON_PORT = "OS::Neutron::Port"
NEUTRON_SUBNET = "OS::Neutron::Subnet"
NEUTRON_ROUTER = "OS::Neutron::Router"
NEUTRON_SUBNET_POOL = "OS::Neutron::SubnetPool"
NEUTRON_FLOATINGIP = "OS::Neutron::FloatingIP"
NEUTRON_SECURITY_GROUP = "OS::Neutron::SecurityGroup"

NOVA_SERVER = "OS::Nova::Server"
NOVA_SERVERGROUP = "OS::Nova::ServerGroup"
NOVA_HYPERVISOR = "OS::Nova::Hypervisor"
NOVA_FLAVOR = "OS::Nova::Flavor"
NOVA_AVAILABILITY_ZONE = "OS::Nova::AvailabilityZone"
NOVA_SECURITY_GROUP = "OS::Nova::SecurityGroup"

SWIFT_ACCOUNT = "OS::Swift::Account"
SWIFT_CONTAINER = 'OS::Swift::Container'
SWIFT_OBJECT = 'OS::Swift::Object'

KEYSTONE_PROJECT = "OS::Keystone::Project"
KEYSTONE_USER = "OS::Keystone::User"

IRONIC_NODE = "OS::Ironic::Node"
IRONIC_PORT = "OS::Ironic::Port"
IRONIC_CHASSIS = "OS::Ironic::Chassis"
